--
-- **************************************************************************
-- **************************************************************************
-- **                                                                      **
-- **                      "Ukady CPLD w praktyce"                        **
-- **                                                                      **
-- **    Project name: BCDCnt                                              **
-- **    Module: bcdcnt (root)                                             **
-- **    Target device: CoolRunner II (XC2C256/VQ100) [ZL14PLD/ZL15PLD]    **
-- **    Tool versions: ISE WebPACK 9.2i                                   **
-- **    Date: BIAYSTOK, stycze 2009                                     **
-- **                                                                      **
-- **                     Napisa : Andrzej Pawluczuk                      **
-- **                                                                      **
-- **************************************************************************
-- **************************************************************************
--
-----------------------------------------------------------------------------
library IEEE ;
use IEEE.STD_LOGIC_1164.ALL ;
use IEEE.STD_LOGIC_ARITH.ALL ;
use IEEE.STD_LOGIC_UNSIGNED.ALL ;

entity BCDCnt is port ( Clk : in std_logic ;
                        Reset : in std_logic ;
                        Enable : in std_logic ;
                        Seg_A : out std_logic ;
                        Seg_B : out std_logic ;
                        Seg_C : out std_logic ;
                        Seg_D : out std_logic ;
                        Seg_E : out std_logic ;
                        Seg_F : out std_logic ;
                        Seg_G : out std_logic ;
                        Seg_K : out std_logic ;
                        DSel0 : out std_logic ;
                        DSel1 : out std_logic ;
                        DSel2 : out std_logic ;
                        DSel3 : out std_logic ) ;
end BCDCnt ;

architecture Behavioral of BCDCnt is

  component FDivider generic ( constant CntSize : integer := 8 ) ;
                     port ( Clk : in std_logic ;
                            Reset : in std_logic ;
                            FDiv : out std_logic ) ;
  end component ;

  component BCD4DigCnt port ( Clk : in std_logic ;
                              Reset : in std_logic ;
                              Enable : in std_logic ;
                              Cnt1 : out std_logic_vector ( 3 downto 0 ) ;
                              Cnt10 : out std_logic_vector ( 3 downto 0 ) ;
                              Cnt100 : out std_logic_vector ( 3 downto 0 ) ;
                              Cnt1000 : out std_logic_vector ( 3 downto 0 ) ) ;
  end component ;

  component Mux4DigDisp port ( DClk : in std_logic ;
                               Reset : in std_logic ;
                               Neg : in std_logic := '0' ;
                               WA : in std_logic := '0' ;
                               Tetr0 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr1 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr2 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr3 : in std_logic_vector ( 3 downto 0 ) ;
                               Dot   : in std_logic_vector ( 3 downto 0 ) ;
                               Seg_A : out std_logic ;
                               Seg_B : out std_logic ;
                               Seg_C : out std_logic ;
                               Seg_D : out std_logic ;
                               Seg_E : out std_logic ;
                               Seg_F : out std_logic ;
                               Seg_G : out std_logic ;
                               Seg_K : out std_logic ;
                               DSel0 : out std_logic ;
                               DSel1 : out std_logic ;
                               DSel2 : out std_logic ;
                               DSel3 : out std_logic ) ;
  end component ;

  signal Cnt1 : std_logic_vector ( 3 downto 0 ) ;
  signal Cnt10 : std_logic_vector ( 3 downto 0 ) ;
  signal Cnt100 : std_logic_vector ( 3 downto 0 ) ;
  signal Cnt1000 : std_logic_vector ( 3 downto 0 ) ;
  signal DClk : std_logic ;
  signal FClk : std_logic ;
  signal CClk : std_logic ;
  signal FDiv : std_logic_vector ( 11 downto 0 ) ;

begin
  FDivInstance : FDivider generic map ( CntSize => 3 )
                          port map ( Clk => Clk ,
                                     Reset => Reset ,
                                     FDiv => FClk ) ;

  ClkGenInstance : process ( FClk , Reset )
  begin
    if Reset = '0' then
      FDiv <= ( others => '0' ) ;
    else
      if FClk'event and FClk = '0' then
        FDiv <= FDiv + 1 ;
      end if ;
    end if ;
  end process ;

  DClk <= FDiv ( 3 ) ;

  CClk <= FDiv ( 11 ) ;

  BCDCntInstance : BCD4DigCnt port map ( Clk => CClk ,
                                         Reset => Reset ,
                                         Enable => Enable ,
                                         Cnt1 => Cnt1 ,
                                         Cnt10 => Cnt10 ,
                                         Cnt100 => Cnt100 ,
                                         Cnt1000 => Cnt1000 ) ;

  MuxDisplInstance : Mux4DigDisp port map ( DClk => DClk ,
                                            Reset => Reset ,
                                            Neg => '1' ,
                                            WA => '1' ,
                                            Tetr0 => Cnt1 ,
                                            Tetr1 => Cnt10 ,
                                            Tetr2 => Cnt100 ,
                                            Tetr3 => Cnt1000 ,
                                            Dot => "1111" ,
                                            Seg_A => Seg_A ,
                                            Seg_B => Seg_B ,
                                            Seg_C => Seg_C ,
                                            Seg_D => Seg_D ,
                                            Seg_E => Seg_E ,
                                            Seg_F => Seg_F ,
                                            Seg_G => Seg_G ,
                                            Seg_K => Seg_K ,
                                            DSel0 => DSel0 ,
                                            DSel1 => DSel1 ,
                                            DSel2 => DSel2 ,
                                            DSel3 => DSel3 ) ;

end Behavioral ;
