--
-- **************************************************************************
-- **************************************************************************
-- **                                                                      **
-- **                      "Ukady CPLD w praktyce"                        **
-- **                                                                      **
-- **            Module: LCD2DigDisp (przydatny komponent)                 **
-- **            Target device: XILINX CPLD                                **
-- **            Tool versions: ISE WebPACK 9.2i                           **
-- **            Date: BIAYSTOK, stycze 2009                             **
-- **                                                                      **
-- **                     Napisa : Andrzej Pawluczuk                      **
-- **                                                                      **
-- **************************************************************************
-- **************************************************************************
--
-----------------------------------------------------------------------------
library IEEE ;
use IEEE.STD_LOGIC_1164.ALL ;
use IEEE.STD_LOGIC_ARITH.ALL ;
use IEEE.STD_LOGIC_UNSIGNED.ALL ;

entity LCD2DigDisp is port ( Clk : std_logic ;
                             MClk : in std_logic ;
                             Reset : in std_logic ;
                             Tetr0 : in std_logic_vector ( 3 downto 0 ) ;
                             Tetr1 : in std_logic_vector ( 3 downto 0 ) ;
                             Dot : in std_logic_vector ( 1 downto 0 ) ;
                             LCD_BP : out std_logic ;
                             LCD1_A : out std_logic ;
                             LCD1_B : out std_logic ;
                             LCD1_C : out std_logic ;
                             LCD1_D : out std_logic ;
                             LCD1_E : out std_logic ;
                             LCD1_F : out std_logic ;
                             LCD1_G : out std_logic ;
                             LCD1_DP : out std_logic ;
                             LCD10_A : out std_logic ;
                             LCD10_B : out std_logic ;
                             LCD10_C : out std_logic ;
                             LCD10_D : out std_logic ;
                             LCD10_E : out std_logic ;
                             LCD10_F : out std_logic ;
                             LCD10_G : out std_logic ;
                             LCD10_DP : out std_logic ) ;
end LCD2DigDisp ;

architecture Behavioral of LCD2DigDisp is

  component UHexConv port ( A : in std_logic ;
                            B : in std_logic ;
                            C : in std_logic ;
                            D : in std_logic ;
                            Neg : in std_logic ;
                            Seg_A : out std_logic ;
                            Seg_B : out std_logic ;
                            Seg_C : out std_logic ;
                            Seg_D : out std_logic ;
                            Seg_E : out std_logic ;
                            Seg_F : out std_logic ;
                            Seg_G : out std_logic ) ;
  end component ;

  signal LCD1 : std_logic_vector ( 6 downto 0 ) ;
  signal LCD10 : std_logic_vector ( 6 downto 0 ) ;
  signal RegLCD1 : std_logic_vector ( 3 downto 0 ) ;
  signal RegLCD10 : std_logic_vector ( 3 downto 0 ) ;
  signal DotReg : std_logic_vector ( 1 downto 0 ) ;

begin
  LCDRegInstance : process ( Clk , Reset )
  begin
    if Reset = '0' then
      RegLCD1 <= ( others => '0' ) ;
      RegLCD10 <= ( others => '0' ) ;
      DotReg <= ( others => '0' ) ;
    else
      if Clk'event and Clk = '0' then
        RegLCD1 <= Tetr0 ;
        RegLCD10 <= Tetr1 ;
        DotReg <= Dot ;
      end if ;
    end if ;
  end process ;

  Conv1Instance : UHexConv port map ( A => RegLCD1 ( 0 ) ,
                                      B => RegLCD1 ( 1 ) ,
                                      C => RegLCD1 ( 2 ) ,
                                      D => RegLCD1 ( 3 ) ,
                                      Neg => '1' ,
                                      Seg_A => LCD1 ( 0 ) ,
                                      Seg_B => LCD1 ( 1 ) ,
                                      Seg_C => LCD1 ( 2 ) ,
                                      Seg_D => LCD1 ( 3 ) ,
                                      Seg_E => LCD1 ( 4 ) ,
                                      Seg_F => LCD1 ( 5 ) ,
                                      Seg_G => LCD1 ( 6 ) ) ;
  Conv10Instance : UHexConv port map ( A => RegLCD10 ( 0 ) ,
                                       B => RegLCD10 ( 1 ) ,
                                       C => RegLCD10 ( 2 ) ,
                                       D => RegLCD10 ( 3 ) ,
                                       Neg => '1' ,
                                       Seg_A => LCD10 ( 0 ) ,
                                       Seg_B => LCD10 ( 1 ) ,
                                       Seg_C => LCD10 ( 2 ) ,
                                       Seg_D => LCD10 ( 3 ) ,
                                       Seg_E => LCD10 ( 4 ) ,
                                       Seg_F => LCD10 ( 5 ) ,
                                       Seg_G => LCD10 ( 6 ) ) ;

  LCD_BP <= MClk ;
  LCD1_A <= LCD1 ( 0 ) xor MClk ;
  LCD1_B <= LCD1 ( 1 ) xor MClk ;
  LCD1_C <= LCD1 ( 2 ) xor MClk ;
  LCD1_D <= LCD1 ( 3 ) xor MClk ;
  LCD1_E <= LCD1 ( 4 ) xor MClk ;
  LCD1_F <= LCD1 ( 5 ) xor MClk ;
  LCD1_G <= LCD1 ( 6 ) xor MClk ;
  LCD1_DP <= ( not DotReg ( 0 ) ) xor MClk ;
  LCD10_A <= LCD10 ( 0 ) xor MClk ;
  LCD10_B <= LCD10 ( 1 ) xor MClk ;
  LCD10_C <= LCD10 ( 2 ) xor MClk ;
  LCD10_D <= LCD10 ( 3 ) xor MClk ;
  LCD10_E <= LCD10 ( 4 ) xor MClk ;
  LCD10_F <= LCD10 ( 5 ) xor MClk ;
  LCD10_G <= LCD10 ( 6 ) xor MClk ;
  LCD10_DP <= ( not DotReg ( 1 ) ) xor MClk ;

end Behavioral ;
