--
-- **************************************************************************
-- **************************************************************************
-- **                                                                      **
-- **                      "Ukady CPLD w praktyce"                        **
-- **                                                                      **
-- **    Project name: DispLCD                                             **
-- **    Module: DispLCD (root)                                            **
-- **    Target device: CoolRunner II (XC2C256/VQ100) [ZL14PLD/ZL15PLD]    **
-- **    Tool versions: ISE WebPACK 9.2i                                   **
-- **    Date: BIAYSTOK, stycze 2009                                     **
-- **                                                                      **
-- **                     Napisa : Andrzej Pawluczuk                      **
-- **                                                                      **
-- **************************************************************************
-- **************************************************************************
--
-----------------------------------------------------------------------------
library IEEE ;
use IEEE.STD_LOGIC_1164.ALL ;
use IEEE.STD_LOGIC_ARITH.ALL ;
use IEEE.STD_LOGIC_UNSIGNED.ALL ;

entity DispLCD is port ( Clk : in std_logic ;
                         Reset : in std_logic ;
                         LCD_BP : out std_logic ;
                         LCD1_A : out std_logic ;
                         LCD1_B : out std_logic ;
                         LCD1_C : out std_logic ;
                         LCD1_D : out std_logic ;
                         LCD1_E : out std_logic ;
                         LCD1_F : out std_logic ;
                         LCD1_G : out std_logic ;
                         LCD1_DP : out std_logic ;
                         LCD10_A : out std_logic ;
                         LCD10_B : out std_logic ;
                         LCD10_C : out std_logic ;
                         LCD10_D : out std_logic ;
                         LCD10_E : out std_logic ;
                         LCD10_F : out std_logic ;
                         LCD10_G : out std_logic ;
                         LCD10_DP : out std_logic ) ;
end DispLCD ;

architecture Behavioral of DispLCD is

  constant FDivCntSizeParam : integer := 3 ;

  component FDivider is generic ( constant CntSize : integer := 8 ) ;
                     port ( Clk : in std_logic ;
                            Reset : in std_logic ;
                            FDiv : out std_logic ) ;
  end component ;

  component BCD2DigCnt port ( Clk : in std_logic ;
                              Reset : in std_logic ;
                              Enable : in std_logic ;
                              Cnt1 : out std_logic_vector ( 3 downto 0 ) ;
                              Cnt10 : out std_logic_vector ( 3 downto 0 ) ) ;
  end component ;

  component LCD2DigDisp port ( Clk : std_logic ;
                               MClk : in std_logic ;
                               Reset : in std_logic ;
                               Tetr0 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr1 : in std_logic_vector ( 3 downto 0 ) ;
                               Dot : in std_logic_vector ( 1 downto 0 ) ;
                               LCD_BP : out std_logic ;
                               LCD1_A : out std_logic ;
                               LCD1_B : out std_logic ;
                               LCD1_C : out std_logic ;
                               LCD1_D : out std_logic ;
                               LCD1_E : out std_logic ;
                               LCD1_F : out std_logic ;
                               LCD1_G : out std_logic ;
                               LCD1_DP : out std_logic ;
                               LCD10_A : out std_logic ;
                               LCD10_B : out std_logic ;
                               LCD10_C : out std_logic ;
                               LCD10_D : out std_logic ;
                               LCD10_E : out std_logic ;
                               LCD10_F : out std_logic ;
                               LCD10_G : out std_logic ;
                               LCD10_DP : out std_logic ) ;
  end component ;

  signal Cnt1 : std_logic_vector ( 3 downto 0 ) ;
  signal Cnt10 : std_logic_vector ( 3 downto 0 ) ;
  signal FClk : std_logic ;
  signal CClk : std_logic ;
  signal MClk : std_logic ;
  signal FDiv : std_logic_vector ( 11 downto 0 ) ;

begin
  FDivInstance : FDivider generic map ( CntSize => FDivCntSizeParam )
                          port map ( Clk => Clk ,
                                     Reset => Reset ,
                                     FDiv => FClk ) ;

  ClkGenInstance : process ( FClk , Reset )
  begin
    if Reset = '0' then
      FDiv <= ( others => '0' ) ;
    else
      if FClk'event and FClk = '0' then
        FDiv <= FDiv + 1 ;
      end if ;
    end if ;
  end process ;

  MClk <= FDiv ( 6 ) ;
  CClk <= FDiv ( 11 ) ;

  CntInstance : BCD2DigCnt port map ( Clk => CClk ,
                                      Reset => Reset ,
                                      Enable => '1' ,
                                      Cnt1 => Cnt1 ,
                                      Cnt10 => Cnt10 ) ;

  LCDMuxInstance : LCD2DigDisp port map ( Clk => FClk ,
                                          MClk => MClk ,
                                          Reset => Reset ,
                                          Tetr0 => Cnt1 ,
                                          Tetr1 => Cnt10 ,
                                          Dot => "11" ,
                                          LCD_BP => LCD_BP ,
                                          LCD1_A => LCD1_A ,
                                          LCD1_B => LCD1_B ,
                                          LCD1_C => LCD1_C ,
                                          LCD1_D => LCD1_D ,
                                          LCD1_E => LCD1_E ,
                                          LCD1_F => LCD1_F ,
                                          LCD1_G => LCD1_G ,
                                          LCD1_DP => LCD1_DP ,
                                          LCD10_A => LCD10_A ,
                                          LCD10_B => LCD10_B ,
                                          LCD10_C => LCD10_C ,
                                          LCD10_D => LCD10_D ,
                                          LCD10_E => LCD10_E ,
                                          LCD10_F => LCD10_F ,
                                          LCD10_G => LCD10_G ,
                                          LCD10_DP => LCD10_DP ) ;

end Behavioral ;
